/*---------------------------------------------------
                  t <-- CHKCONS(n,s,N,S,L,a; sp,Np,Sp,Lp)

Check the contraints.

Input:
   n   is the number of projection factors.
   s   is the current signature.
   N   is the numbers of unconnected multiplicities.
   S   is the information about a strip to be considered.
   L   is the information about a line to be considered
   a   is the number of branches proposed.

Output:
   t   is 1 if ok, 0 otherise.
   sp, Np, Sp, Lp are updated ones. (when t = 1.)

----------------------------------------------------*/
#include "saclib.h"
#include "qepcad.h"

Word CHKCONS(n,s,N,S,L,a,sp_,Np_,Sp_,Lp_)
       Word n,s,N,S,L,a;
       Word *sp_, *Np_, *Sp_, *Lp_;
{
       Word ss,sl,Ms,Ms1,Ml,Ml1,N1,p,m,i,M,M1;
       Word sp,Np,Sp,Lp,T,T1,D;

Step1: /* Not enough branches? */
       if (a > LENGTH(S)) return(0);

Step2: /* Too many unconnected multiplicities? */
       if (FIRST(FIRST(L)) - a > FIRST(N)) return(0);

Step3: /* Signature mismatch? */
       if (a > 0) 
         ss = THIRD(LELTI(S,a));
       else
         ss = s;
       sl = THIRD(FIRST(L));
       if (!EQUAL(ss,sl)) return(0);

Step4: /* Stripe multiplicites. */
       Ms = NIL;
       for (i=1; i <= n; i++)
         Ms = COMP(0,Ms);
       for (i=1; i <= a; i++)
         {
         M = SECOND(LELTI(S,i));
         while (M != NIL)
           {
           ADV(M,&M1,&M);
           FIRST2(M1,&p,&m);
           SLELTI(Ms,p, LELTI(Ms,p) + m);
           }
         }

Step5: /* Line multiplicites. */
       Ml = NIL;
       for (i=1; i <= n; i++)
         Ml = COMP(0,Ml);
       M = SECOND(FIRST(L));
       while (M != NIL)
         {
         ADV(M,&M1,&M);
         FIRST2(M1,&p,&m);
         SLELTI(Ml,p,m);
         }

Step6: /* Check multiplicites. */
       N = RED(N);
       Np = NIL;
       T = 0;
       while (Ms != NIL)
         {
         ADV(Ms,&Ms1,&Ms);
         ADV(Ml,&Ml1,&Ml);
         ADV(N,&N1,&N);
         D = Ml1 - Ms1;
         if (D < 0) return(0);
         if (ODD(D)) return(0);
         T1 = N1 - D;
         if (T1 < 0) return(0);
         T = T + T1;
         Np = COMP(T,Np);
         }
       Np = INV(Np);
       Np = COMP(T,Np);

Step7: /* Update. */
       sp = ss;
       Sp = REDI(S,a);
       Lp = RED(L);

Step8: /* Return. */
       *sp_ = sp;
       *Np_ = Np;
       *Sp_ = Sp;
       *Lp_ = Lp;
       return(1);
}




