/*======================================================================
                      MVLMA(T_t,T_f,L_t,L_f,W,T; L)

Multiple-Valued Logic Minimization Algorithm.

\Input
  \parm{Tt} is a sum-of-product for the TRUE-set of a logic function.
  \parm{Tf} is a sum-of-proudct for the FALSE-set of the logic function.
  \parm{Lt} is the length of Tt.
  \parm{Lf} is the length of Tf.
  \parm{W}  is the length of a product.
  \parm{T}  is an array to hold the products. (Physical size = L_t).
                                              Is this sufficient?
                                              Let me think about it......

\Output
  \parm{T} is a simple cover for the logic function.
  \parm{L} is the length of T.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void MVLMA(T_t,T_f,L_t,L_f,W,T,L_)
       RelOpsTables T_t,T_f,T;
       Word L_t,L_f,W,*L_;
{
       Word L,i,Nr;
       MarksTable M;

Step1: /* Clear the marks. */
       for (i = 1; i <= L_t; i++) 
         T_t[i][0] = 0;
       for (i = 1; i <= L_f; i++) 
         T_f[i][0] = 0;

Step2: /* Generalte prime implicants. */
                                /*Int*/ Thss = ACLOCK();
                                /*Int*/ Ths = Thss;

       M = MARKTALLOC(L_t);     /* Is this sufficient?? */

       GENPIMP(T_t,T_f,L_t,L_f,W,T,M,&Nr);
                                /*Int*/ Ths = ACLOCK() - Ths;
                                /*Int*/ TMGENIMP = Ths;

Step3: /* Select. */
                                /*Int*/ Ths = ACLOCK();
       SELPIMP(M,L_t,Nr);
                                /*Int*/ Ths = ACLOCK() - Ths;
                                /*Int*/ TMSELIMP = Ths;
                                /*Int*/ Thss = ACLOCK() - Thss;
                                /*Int*/ TMMVLMASF = Thss;

Step4: /* Pack. */
       L = 0;
       for (i = 1; i <= Nr; i++) 
         if (M[i][0] == ESSENTIAL)
           {
           L++;
           T[L] = T[i];
           }

Return: /* Prepare for return. */
       *L_ = L;
       return;
}

