/*======================================================================
                      NEXTPRD(n,vs,Ss_r,Ss_s,Rs_c,Rs_s; v,S_r,S_s,R_c,R_s,q)

Next Product.

\Input
  \parm{n}   is the number of variables.
  \parm{v*}  is a product.
  \parm{S*r} is the slot arragment for relational operators.
  \parm{S*s} is the slot arrgement for strict relational operators.
  \parm{R*c} is the compound relational operators arrangement.
  \parm{R*s} is the strict relational operators arrangement.
  
\Output
  \parm{v}  is a next product if successful.
  \parm{Sr} is the slot arragment for relational operators.
  \parm{Ss} is the slot arrgement for strict relational operators.
  \parm{Rc} is the compound relational operators arrangement.
  \parm{Rs} is the strict relational operators arrangement.
  \parm{q}  is 1 if successful, 0 otherwise.

\Side 
  All input variables but $n$ are modified.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void NEXTPRD(n,vs,Ss_r,Ss_s,Rs_c,Rs_s, v_,S_r_,S_s_,R_c_,R_s_,q_)
       Word n,vs,Ss_r,Ss_s,Rs_c,Rs_s, *v_,*S_r_,*S_s_,*R_c_,*R_s_,*q_;
{
       Word R_c,R_s,S_r,S_s,c,q,r,s,v;
       /* hide c,q,r,s; */

Step1: /* Setup. */
       v = vs; S_r = Ss_r; S_s = Ss_s; R_c = Rs_c; R_s = Rs_s;
       r = LENGTH(S_r); s = LENGTH(S_s); c = r - s;

Step2: /* More strict relational operator arrangement? */
       NEXTLEXI(s,R_s,&R_s,&q);
       if (q == 0) goto Step3;
       FILLIN(n,v,S_r,S_s,R_c,R_s,&v);
       goto Return;

Step3: /* More compound relational operator arrangement? */
       NEXTLEXI(c,R_c,&R_c,&q);
       if (q == 0) goto Step4;
       FIRSTLEXI(s,&R_s);
       FILLIN(n,v,S_r,S_s,R_c,R_s,&v);
       goto Return;

Step4: /* More slot arrangement for strict relational operators? */
       NEXTCOMB(r,s,S_s,&S_s,&q);
       if (q == 0) goto Step5;
       FIRSTLEXI(c,&R_c);
       FIRSTLEXI(s,&R_s);
       FILLIN(n,v,S_r,S_s,R_c,R_s,&v);
       goto Return;

Step5: /* More slot arrangment for relational operators? */
       NEXTCOMB(n,r,S_r,&S_r,&q);
       if (q == 0) goto Step6;
       FIRSTCOMB(r,s,&S_s);
       FIRSTLEXI(c,&R_c);
       FIRSTLEXI(s,&R_s);
       FILLIN(n,v,S_r,S_s,R_c,R_s,&v);
       goto Return;

Step6: /* More possible number of strict relational operators? */
       s = s + 1; if (s <= r) q = 1; else q = 0;
       if (q == 0) goto Step7;
       c = r - s;
       FIRSTCOMB(n,r,&S_r);
       FIRSTCOMB(r,s,&S_s);
       FIRSTLEXI(c,&R_c);
       FIRSTLEXI(s,&R_s);
       FILLIN(n,v,S_r,S_s,R_c,R_s,&v);
       goto Return;

Step7: /* More possible number of relational operators? */
       r = r + 1; if (r <= n) q = 1; else q = 0;
       if (q == 0) goto Step8;
       s = 0;
       c = r;
       FIRSTCOMB(n,r,&S_r);
       FIRSTCOMB(r,s,&S_s);
       FIRSTLEXI(c,&R_c);
       FIRSTLEXI(s,&R_s);
       FILLIN(n,v,S_r,S_s,R_c,R_s,&v);
       goto Return;

Step8: /* No more product. */
       goto Return;

Return: /* Prepare for return. */
       *v_ = v;
       *S_r_ = S_r;
       *S_s_ = S_s;
       *R_c_ = R_c;
       *R_s_ = R_s;
       *q_ = q;
       return;
}
