/*======================================================================
                         SELPIMP(M,Nc,Nr)

Select Prime Implicants.

\Input
  \parm{M}  is an array of marks associated with the prime implicants
  \parm{Nc} is the length of an element of M.
  \parm{Nr} is the length of the M.
           
\Output
  \parm{M}  is modified so that the selected rows (prime implicants)
            are marked.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"
#define ALIVE      0
#define DEAD       1
#define ESSENTIAL  2

Word SELPIMP(M,Nc,Nr)
       MarksTable  M;
       Word Nc,Nr;
{
       Word i,j,ip,jp,t,k;

Step1: /* Setup. */
       M[0] = MARKALLOC(Nc);
       for (j = 1; j <= Nc; j++)
         M[0][j] = ALIVE;
       for (i = 1; i <= Nr; i++)
         M[i][0] = ALIVE;
       RNc = Nc;
        
Step2: /* Remove covered prime implicants. */
       for (i = 1; i <= Nr-1; i++)
         {
         if (M[i][0] != ALIVE) continue;
         for (ip = i + 1; ip <= Nr; ip++)
           {
           if (M[ip][0] != ALIVE) continue;
           if (COVERS(M,i,ip,Nc)) 
             {
             M[ip][0] = DEAD; 
             t = 1;
             }
           else 
             {
             if (COVERS(M,ip,i,Nc))
               {
               M[i][0] = DEAD;
               t = 1;
	       }
	     }
	   }
         }  

Step3: /* Collect the essential prime implicants. */
                                  /*Int*/ if (TCPIMPTBL == 'y') TRPIMPTBL(M,Nc,Nr);
       t = 0;
       for (j = 1; j <= Nc; j++)
         {
         if (M[0][j] != ALIVE) continue;
         k = 0;
         for (i = 1; i <= Nr; i++)
           {
           if (M[i][0] != ALIVE) continue;
           if (M[i][j] == 1) 
             { 
             k = k + 1; 
             if (k >= 2) break; 
             ip = i; 
             }
           }
         if (k != 1) continue;

         M[ip][0] = ESSENTIAL; 
                                /*Int*/ NMSELIMP = NMSELIMP + 1;
                                /*Int*/ if (TCEPIMP == 'y') TREPIMP(ip);
         for (jp = 1; jp <= Nc; jp++)
           {
           if (M[ip][jp] == 1 && M[0][jp] == ALIVE) 
             { 
             RNc--; 
             M[0][jp] = DEAD; 
             }
                                /*Int*/ if (TCPIMPTBL == 'y') TRPIMPTBL(M,Nc,Nr);
	   }
         t = 1; 
         }

       if (t == 0) 
         {
         /* No essential prime implicants. 
            Choose any prime implicant as if eseential.  
            It not an optimal method.
            Needs to be improved by a branch-and-bound method. */
         for (i = i; i <= Nr; i++)
           if (M[i][0] == ALIVE) { ip = i; break; }

         M[ip][0] = ESSENTIAL; 
                                /*Int*/ NMSELIMP = NMSELIMP + 1;
                                /*Int*/ if (TCEPIMP == 'y') TREPIMP(ip);
         for (jp = 1; jp <= Nc; jp++)
           {
           if (M[ip][jp] == 1 && M[0][jp] == ALIVE) 
             { 
             RNc--; 
             M[0][jp] = DEAD; 
             }
                              /*Int*/ if (TCPIMPTBL == 'y') TRPIMPTBL(M,Nc,Nr);
           }
         }
       if (RNc == 0) goto Return; 
       goto Step2;

Return: /* Prepare for return. */
       return;
}



