/*======================================================================
                      Fh <- SIMPLIFY(T_t,T_f,L_t,L_f,W,F,P,f)

Simplify.

\Input
  \parm{Tt} is an array of distinct projection signatures. 
  \parm{Tf} is an array of distinct projection signatures.
            $T_t$ and $T_f$ are disjoint.
  \parm{Lt} is the length of Tt.
  \parm{Lf} is the length of Tf.
  \parm{W   is the length of a signature.
  \parm{F}  is the normalized quantifier-free part of 
            the input formula.
  \parm{P}  is the list $(P_1,\ldots,P_r)$ where $P_i$
            is the list of the $i$--level projection factors.
  \parm{f}  is the number of free variables.

\Output
  \parm{F^} is a quantifier-free formula.
            See the dissertation for the detail.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

Word SIMPLIFY(T_t,T_f,L_t,L_f,W,F,P,f)
       Word L_t,L_f,W,F,P,f;
       RelOpsTable T_t, T_f;
{
       Word Fh,Fp,L;
       RelOpsTable T;

Step1: /* Simplify. */
       if      (L_t == 0) 
          Fp = LIST4(NEOP,0,0,NIL); 
       else if (L_f == 0)
          Fp = LIST4(EQOP,0,0,NIL); 
       else
         {
         T = RELTALLOC(L_t);
         MVLMA(T_t,T_f,L_t,L_f,W,T,&L);
         Fp = QFFFSOP(T,L,W,P,f);
         }
       Fh = REPLACE(f,F,Fp);

Return: /* Prepare for return. */
       return(Fh);
}


