/*======================================================================
                      SOLUTION(D,F,P,f; t,F_e,F_n,F_s)

Solution Formula Construction Phase.
 
\Input
  \parm{D} is a truth--invariant partial CAD.
  \parm{F} is the normalized quantifier-free  part of the input formula.
  \parm{P} is the list~$(P_1,\ldots,P_r)$, 
           where $P_i$ is the list of
           the $i$--level projection factors.
  \parm{f} is the number of free variables in the input formula.
 
\Output
  \parm{t}  is either \cn{equ} or \cn{inequ}.
  \parm{Fe} is a quantifier-free formula equivalent to
            the input formula if  $t$ is \cn{equ},
            otherwise $F_e$ is undefined.
  \parm{Fn} is a quantifier-free formula necessary for
            the input formula if $t$ is \cn{inequ},
            otherwise $F_n$ is undefined.
  \parm{Fs} is a quantifier-free formula sufficient for
            the input formula if $t$ is \cn{inequ},
            otherwise $F_s$ is undefined.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void SOLUTION(D,F,P,f, t_,F_e_,F_n_,F_s_)
       Word D,F,P,f, *t_,*F_e_,*F_n_,*F_s_;
{
       Word F_e,F_n,F_s,T_c,Ths,Thss,t,L_t,L_f,W,Lp_t,Lp_f;
       RelOpsTable T_t, T_f,Tp_t,Tp_f;

Step1: /* Decision Problem. */
                                /*Int*/ Thss = ACLOCK();
       if (f > 0) goto Step2;
       t = EQU;
       if (LELTI(D,TRUTH) == TRUE)
         F_e = LIST4(EQOP,0,0,NIL);
       else
         F_e = LIST4(NEOP,0,0,NIL);
                               /*Int*/ Thss = ACLOCK() - Thss;
                               /*Int*/ TMSOLUTION = Thss;
       goto Return;

Step2: /* Signature tables. */
                               /*Int*/ Ths = ACLOCK();
       W = LENPROJF(P,f);
       LENSIGTBL(D,f,0,0,&L_t,&L_f);
       T_t = RELTALLOC(L_t);
       T_f = RELTALLOC(L_f);
       SIGTBL(D,f,T_t,T_f,0,0,&L_t,&L_f);
       RMDUPA(T_t,L_t,W,&L_t);
       RMDUPA(T_f,L_f,W,&L_f);
                               /*Int*/ Ths = ACLOCK() - Ths;
                               /*Int*/ TMSIGTBL = Ths;
                               /*Int*/ NMPRPT = L_t;
                               /*Int*/ NMPRPF = L_f;

Step3: /* Any indistinguishable cell? */
                               /*Int*/ Ths = ACLOCK();
       N_c = SIGINS(T_t,T_f,L_t,L_f,W);
                               /*Int*/ Ths = ACLOCK() - Ths;
                               /*Int*/ TMSIGINS = Ths;
                               /*Int*/ NMPRPC = N_c;
       if (N_c == 0)
          t = EQU;
       else
         { 
         t = INEQU; 
         Lp_t = L_t - N_c;  Tp_t = SIGSUB(T_t,L_t,Lp_t); 
         Lp_f = L_f - N_c;  Tp_f = SIGSUB(T_f,L_f,Lp_f); 
         goto Step5; 
         }

Step4: /* $F_e$. */
       F_e = SIMPLIFY(T_t,T_f,L_t,L_f,W,F,P,f);
                               /*Int*/ Thss = ACLOCK() - Thss;
                               /*Int*/ TMSOLUTION = Thss;
       goto Return;

Step5: /* $F_n$. */
       F_n = SIMPLIFY(T_t,Tp_f,L_t,Lp_f,W,F,P,f);

Step6: /* $F_s$. */
       F_s = SIMPLIFY(Tp_t,T_f,Lp_t,L_f,W,F,P,f);
                               /*Int*/ Thss = ACLOCK() - Thss;
                               /*Int*/ TMSOLUTION = Thss;
       goto Return;

Return: /* Prepare for return. */
       *t_ = t;
       *F_e_ = F_e;
       *F_n_ = F_n;
       *F_s_ = F_s;
       return;
}



