/*======================================================================
                      F <- NORMAF(A)

Normalize Atomic Formula.

\Input
  \parm{A} is an atomic formula.

\Output
  \parm{F} is a quantifier--free formula equivalent to \v{A}
           such that all the atomic formulas in \v{F} are 
           normalized.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

Word NORMAF(A)
       Word A;
{
       Word F,I,L,L_i,Lh,Lh_i,P,P_i,Ph_i,T,c,e_i,r,rh_i,s;
       /* hide rh_i,s; */

Step1: /* Get the components. */
       FIRST4(A,&T,&P,&r,&I);

Step2: /* \v{P} = 0. */
       if (P != 0) goto Step3;
       switch (T)
         {
         case EQOP: F = LIST4(EQOP,0,0,NIL); break;
         case GEOP: F = LIST4(EQOP,0,0,NIL); break;
         case LEOP: F = LIST4(EQOP,0,0,NIL); break;
         case GTOP: F = LIST4(NEOP,0,0,NIL); break;
         case LTOP: F = LIST4(NEOP,0,0,NIL); break;
         case NEOP: F = LIST4(NEOP,0,0,NIL);
         break; }
       goto Return;

Step3: /* Facor \v{P}. */
       IPFACDB(r,P,&s,&c,&L);

Step4: /* Adjust \v{T}. */
       if (s < 0) T = NEGRLOP(T);

Step5: /* \v{P} is an integer. */
       if (L != NIL) goto Step6;
       switch (T)
         {
         case NEOP: F = LIST4(EQOP,0,0,NIL); break;
         case GTOP: F = LIST4(EQOP,0,0,NIL); break;
         case GEOP: F = LIST4(EQOP,0,0,NIL); break;
         case EQOP: F = LIST4(NEOP,0,0,NIL); break;
         case LTOP: F = LIST4(NEOP,0,0,NIL); break;
         case LEOP: F = LIST4(NEOP,0,0,NIL);
         break; }
       goto Return;

Step6: /* Simplify the representation of the polys in \v{L}. */
       Lh = NIL;
       while (L != NIL)
         {
         ADV(L,&L_i,&L);
         FIRST2(L_i,&e_i,&P_i);
         PSIMREP(r,P_i,&rh_i,&Ph_i);
         Lh_i = LIST3(e_i,rh_i,Ph_i);
         Lh = COMP(Lh_i,Lh);
         }
       Lh = INV(Lh);

Step7: /* Expand. */
       switch (T)
         {
         case EQOP: F = EXPAFEQ(Lh); break;
         case GTOP: F = EXPAFGT(Lh); break;
         case LTOP: F = EXPAFLT(Lh); break;
         case NEOP: F = LIST2(NOTOP,EXPAFEQ(Lh)); break;
         case LEOP: F = LIST2(NOTOP,EXPAFGT(Lh)); break;
         case GEOP: F = LIST2(NOTOP,EXPAFLT(Lh)); break;
         }
       goto Return;

Return: /* Prepare for return. */
       return(F);
}
