/*======================================================================
                      AFUPGC(M,A,B; C,Ab,Bb)

Algebraic number field univariate polynomial greatest common divisor
and cofactors.  

Inputs
    M : in Z[x], the minimal polynomial for an algebraic number alpha.
  A,B : in Q(alpha)[x].

Outputs
    C : in Q(alpha)[x], C is monic, C = gcd(A,B).
   Ab : in Q(alpha)[x]. If C not zero then As = A/C else As = 0.
   Bb : in Q(alpha)[x]. If C not zero then Bs = B/C else Bs = 0.
======================================================================*/
#include "saclib.h"

void AFUPGC(M,A,B, C_,Ab_,Bb_)
       Word M,A,B, *C_,*Ab_,*Bb_;
{
       Word Ab,Bb,C,As,Bs,as,bs,P,R,e1,C1,Cp,Q,Mb;

Step1: /* A = B = 0. */
       if (A == 0 && B == 0)
         {
         C = 0;
         Ab = 0;
         Bb = 0;
         goto Return;
         }

Step2: /* A = 0 or B = 0. */
       if (A == 0)
         {
         C = AFPMON(1,M,B);
         Ab = 0;
         Bb = PINV(0,PLDCF(B),1);
         goto Return;
         }
       if (B == 0)
         {
         C = AFPMON(1,M,A);
         Bb = 0;
         Ab = PINV(0,PLDCF(A),1);
         goto Return;
         }

Step3: /* Get the integral polynomials. */
       As = AFPICR(1,A);
       Bs = AFPICR(1,B);
       IPSRP(2,As,&as,&As);
       IPSRP(2,Bs,&bs,&Bs);

Step4: /* Compute the subresultant prs. */
       if (PDEG(As) >= PDEG(Bs))
         P = IPSPRS(2,As,Bs);
       else
         P = IPSPRS(2,Bs,As);

Step5: /* Search for the gcd. */
       Mb = RPFIP(1,M);
       P = INV(P);
       while (P != NIL)
         {
         ADV(P,&C,&P);
         Cp = NIL;
         while (C != NIL)
           {
           ADV2(C,&e1,&C1,&C);
           RPQR(1,RPFIP(1,C1),Mb,&Q,&C1); 
           C1 = AFCR(C1);
           if (C1 != 0)
             Cp = COMP2(C1,e1,Cp);
           }
         C = INV(Cp);
         if (C != NIL)
           goto Step6;
         }
       FAIL("AFUPGC","Should never reach here.");       

Step6: /* Make it monic. */
       C = AFPMON(1,M,C);

Step7: /* Make up cofactors. */
       AFPQR(1,M,A,C,&Ab,&R);
       AFPQR(1,M,B,C,&Bb,&R);

Return: /* Prepare for return. */
       *C_ = C;
       *Ab_ = Ab;
       *Bb_ = Bb;
       return;
}




