                      V:=GENPIMP(Ht,Hf)
 
[Generate Prime Implicants.
 
\Input
  \parm{Ht} is a sum-of-product. (list of minterms)
  \parm{Hf} is a sum-of-product. (list of minterms)
            They together define a logic function~$L$.
          
\Output
  \parm{V} is a list of 3-tuples $(v,l,T)$ 
           where $v$ is an prime implicant of $L$, $l$ is the length of $v$, and
           $T$ is the list of minterms captured by $v$.
]

include "qepcadGlobals.h"
                   
(1) [Setup.]
    V:=(); 
    m:=LENGTH(Ht); Z:=(); for i=1,...,m do Z:=COMP(i,Z); Z:=INV(Z);
    n := LENGTH(FIRST(Ht));
    FIRSTPRD(n;v,Sr,Ss,Rc,Rs).
    
(2) [Generate the next product.]
    NEXTPRD(n,v,Sr,Ss,Rc,Rs;v,Sr,Ss,Rc,Rs,q);
    if q = 0 then {write "Error GENPIMP: Something is wrong!\n"; stop}.
    
(3) [Does it capture any minterm in $H_f$?] 
    H'f := Hf; 
    while H'f /= () do
      {
      ADV(H'f;H1,H'f);
      if CAPTURE(v,H1) then goto 2;
      }.

(4) [Does it capture any minterm in $H_t$?]
    H't := Ht; T:=(); i:=0;
    while H't /= () do
      {
      ADV(H't;H1,H't); i:=i+1;
      if CAPTURE(v,H1) then T := COMP(i,T);
      };
    T := INV(T);
    if T = () then goto 2.
    
(5) [Is it prime?]
    V' := V;
    while V' /= () do
      {
      ADV(V';V1,V');
      T' := THIRD(V1);
      if ISSUBSET(T,T') then goto 2
      }.

(6) [Register the product.]
    v* := INV(CINV(v));
    V1:=LIST3(v*,PRDLEN(v*),T); V:=COMP(V1,V); Z:=SDIFF(Z,T);
    if Z /= () then goto 2.
    
(7) [Done.]
    V:=INV(V); return ||
