/*======================================================================
                      P <- GENPIMP(H_t,H_f)

Generate Prime Implicants.
 
\Input
  \parm{Ht} is a sum-of-product. (list of minterms)
  \parm{Hf} is a sum-of-product. (list of minterms)
            They together define a logic function~$L$.
          
\Output
  \parm{P} is a tuple $(m,V)$ where
           $m$ is the number of minterms in $H_t$ and
           $V$ is a list of 3-tuples $(v,l,T)$ 
           where $v$ is an prime implicant of $L$, $l$ is the length of $v$, and
           $T$ is the list of minterms captured by $v$.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

Word GENPIMP(H_t,H_f)
       Word H_t,H_f;
{
       Word H1,Hp_f,Hp_t,L,P,R_c,R_s,S_r,S_s,T,Tp,V,V1,Vp,i,m,n,q,v,vs;
       /* hide i,m,n,q; */

Step1: /* Setup. */
       V = NIL;
       m = LENGTH(H_t); L = NIL; for (i = 1; i <= m; i++) L = COMP(i,L); L = INV(L);
       n = LENGTH(FIRST(H_t));
       FIRSTPRD(n,&v,&S_r,&S_s,&R_c,&R_s);

Step2: /* Get the next product. */
       NEXTPRD(n,v,S_r,S_s,R_c,R_s,&v,&S_r,&S_s,&R_c,&R_s,&q);
                                /*Int*/ NMGENPRD = NMGENPRD + 1;
                                /*Int*/ if (TCPROD == 'y') TRPROD(v);

Step3: /* Does it capture any minterm in $H_f$? */
       Hp_f = H_f;
       while (Hp_f != NIL)
         {
         ADV(Hp_f,&H1,&Hp_f);
         if (CAPTURE(v,H1)) goto Step2;
         }

Step4: /* Does it capture any minterm in $H_t$? */
       Hp_t = H_t; T = NIL; i = 0;
       while (Hp_t != NIL)
         {
         ADV(Hp_t,&H1,&Hp_t); i = i + 1;
         if (CAPTURE(v,H1)) T = COMP(i,T);
         }
       T = INV(T);
       if (T == NIL) goto Step2;

Step5: /* Is it prime? */
       Vp = V;
       while (Vp != NIL)
         {
         ADV(Vp,&V1,&Vp);
         Tp = THIRD(V1);
         if (ISSUBSET(T,Tp)) goto Step2;
         }

Step6: /* Register the product. */
       vs = INV(CINV(v));
       V1 = LIST3(vs,PRDLEN(vs),T); V = COMP(V1,V); L = SDIFF(L,T);
                                 /*Int*/ NMGENIMP = NMGENIMP + 1;
                                 /*Int*/ if (TCPIMP == 'y') TRPIMP(vs,T);
       if (L != NIL) goto Step2;

Step7: /* Done. */
       V = INV(V); P = LIST2(m,V);

Return: /* Prepare for return. */
       return(P);
}

