                        H:=SELPIMP(m,V)

[Select Prime Implicants.

\Input
  \parm{m} is the number of the minterms to cover.
  \parm{V} is a list of 3-tuples $(v,l,T)$ 
           where $v$ is an prime implicant, $l$ is the length of $v$, and
           $T$ is the list of minterms captured by $v$.
           
\Output
  \parm{H} is a simple sum-of-product which captures all the minterms.
           (i.e. a short list of implicants  which together still 
           captures all the minterms.)
]

[Local
  \parm{C}  is the marking list for the column.
  \parm{R}  is the marking list for the row.
  \parm{c}  is the running index for columns.
  \parm{r}  is the running index for rows.
  \parm{t}  is 1  if any change of the table occurred.
  \parm{k}  is the number of prime implicants capturing the current minterm.
  \parm{s}  is 1  if there is at least one column alive.
  \parm{f}  is 1  if it is the first time of the essential 
            prime implicant processing.
  \parm{z}  is 1  if a cover is found for the current prime implicant.
  \parm{q}  is 1  if $v^*$ covers $v'$.
]

const ALIVE=1, DEAD=0.

(1) [Setup.]
    n:=LENGTH(V); H:=(); f:=1;
    C:=(); for i=1,...,m do C:=COMP(ALIVE,C); 
    R:=(); for i=1,...,n do R:=COMP(ALIVE,R).

(2) [Collect the essential prime implicants.]
    c:=0; C':=C; t:=0;
    while C' /= () do
      {
      c:=c + 1; ADV(C';C1,C'); 
      if C1 = ALIVE then
        {
        r:=0; R':=R; V':=V; k:=0; 
        while R' /= () /\ k <= 1 do
          {
          r:=r + 1; ADV(R';R1,R'); ADV(V';V1,V'); 
          if R1 = ALIVE then
            {
            FIRST3(V1;v,l,T);
            if INSET(T,c) then {k:= k + 1; V^:=V1; r^:=r};
            };
          };
        if k = 1 then
          {
          SLELT(R,r^,DEAD); FIRST3(V^;v^,l^,T^);
          while T^ /= () do {ADV(T^;T^1,T^);SLELT(C,T^1,DEAD); t:=1};
          H:=COMP(v^,H);  
          };
        };
      }.
           
(3) [Decide the next action.]
    s:=0; C':=C;
    while C' /= ()  /\ s = 0 do {ADV(C';C1,C'); if C1 = ALIVE then s:=1};      
    if s = 0 then  {H:=INV(H); return};
    if f = 1 then  {f:=0; go to 4};
    if t = 0 then  go to 5.
    
(4) [Remove covered implicants.]
    r:=0; R':=R; V':=V; t:=0;
    while R' /= () do
      {
      r:= r + 1; ADV(R';R'1,R'); ADV(V';V'1,V');  
      if R'1 = ALIVE then
        {
        FIRST3(V'1;v',l',T');
        r*:=0; R*:=R; V*:=V;  z:=0; 
        while R* /= () /\ z = 0 do
          {
          r*:=r* + 1; ADV(R*;R*1,R*); ADV(V*;V*1,V*); 
          if R*1 = ALIVE /\ r /= r* then
            {
            FIRST3(V*1;v*,l*,T*);
            if l* <= l' then
              {
              q:=1; c:=0; C':=C;
              while C' /= () /\ q = 1 do
                {
                c:=c + 1; ADV(C';C1,C'); 
                if C1 = ALIVE then if INSET(T',c) /\ ~INSET(T*,c) then  q:=0
                };
              if q = 1 then  z:=1
              }; 
            };          
          };
        if  z = 1 then {SLELT(R,r,DEAD);t:=1};
        }
      };
    if t = 1 then go to 2.    

(5) [Choose a shortest one. NOT Implemented yet.]
    write "SELIMP: Couldn't finish using essential prime  implicants and covering!\n";
    write "        Only the essential prime implicants are returned.\n";
    write "        Soon, I will need to implement this part,\n";
    write "        but I am too tired now.......\n";
    H:=INV(H); return ||
   

