/*======================================================================
                      APEQC(c,k,P)

Apply equational constraints.

Inputs
   c : a cell over which a stack has just been constructed.
   k : the level of c.
   P : the list of projection factors.

Output
   None.

Effect
   If there are no (k+1)-level constraints nothing is done.
Otherwise each sector of the stack and each section of the
stack which fails to satisfy each k-level constraint has
its truth value set to FALSE and its HOWTV field set to
BYEQC.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void APEQC(c,k,P)
        Word c,k,P;
{
        Word c1,E,E1,E11,Ep,Ep1,h,i,j,m,M,Mp,S,t;

Step1: /* Get (k+1)-level equational constraints, and, */
	/*if none, return. */
	E = LELTI(GVEQNCONST,k+1);
	if (E == NIL) return;

Step2: /* Get the stack over c. */
	S = LELTI(c,CHILD);

Step3: /* Get the next cell of the stack, if any, whose */
	/* truth value is undetermined. */
	if (S != NIL)
	   ADV(S,&c1,&S);
	else
	   goto Return;
	t = LELTI(c1,TRUTH);
	if (t != UNDET)
	   goto Step3;

Step4: /* Get the multiplicity list of the cell. */
	M = LELTI(c1,MULSUB);

Step5: /* If a sector, set truth value to FALSE and return
	for next cell. */
	if (M == NIL) {
	   SLELTI(c1,TRUTH,FALSE);
	   SLELTI(c1,HOWTV,BYEQC);
	   goto Step3; }
	   
Step6: /* Initialize the constraint list. */
	Ep = E;

Step7: /* Get the next constraint, if any, and initialize it;*/
	/* else return. */
	if (Ep == NIL)
	   goto Step3;
	else {
	   ADV(Ep,&E1,&Ep);
	   Ep1 = E1; }

Step8: /* Get the level and index of the next element of the */
 	/* constraint, if any; else set truth value of cell to */
	/* FALSE and return for next cell. */
	if (Ep1 == NIL) {
	   SLELTI(c1,TRUTH,FALSE);
           SLELTI(c1,HOWTV,BYEQC);
	   goto Step3; }
	else {
	   ADV(Ep1,&E11,&Ep1);
	   i = SECOND(E11);
	   j = THIRD(E11); }

Step9: /* If the level of the polynomial is less than k+1 */
	/* apply CELLSQ. */
	if (i < k+1) {
	   t = CELLSQ(c,k,i,j);
	   if (t == 1)
	      goto Step7;
           goto Step8; }

Step10: /* The level of the polynomial is k+1; check whether */
	/* its index occurs in the multiplicity list of c. */
	Mp = M;
	while (Mp != NIL) {
	   ADV(Mp,&m,&Mp);
	   if (FIRST(m) == j)
	      goto Step7; }
	goto Step8;

Return: /* Prepare for return. */
        return;
}
