/*======================================================================
                      CONSTRUCT(c,k,f,Ps,As)

Construct a stack.
 
\Input
    \parm{c}  is a candidate cell of~\v{D}.
    \parm{k}  is the level of the cell~\v{c}.
    \parm{f}  is the number of free variables in the input formula.
    \parm{P*} is the list of the $(k+1)$--level projection factors.
    \parm{A*} is the list of all the distinct $(k+1)$--level
              normalized input polynomials.
 
\Output
    A stack is constructed over the cell~\v{c}.
    If \v{A*} $\neq \emptyset$ or $k+1 \leq f$,
         the signature of \v{P*} on each cell in the stack is determined
         and attached to the cell.
======================================================================*/
#include "saclib.h"
#include "qepcad.h"

void CONSTRUCT(c,k,f,Ps,As)
       Word c,k,f,Ps,As;
{
       Word B,E,I,L,M,S,T,Ths,b,s,Pt,At,P1,A1,Ps1,As1;

Step1: /* Extract the projection factors from their attribute lists. */
       Pt = NIL;
       while (Ps != NIL)
         {
         ADV(Ps,&Ps1,&Ps);
         P1 = LELTI(Ps1,PO_POLY);
         Pt = COMP(P1,Pt);
         }
       Pt = INV(Pt);
       Ps = Pt;
       At = NIL;
       while (As != NIL)
         {
         ADV(As,&As1,&As);
         A1 = LELTI(As1,PO_POLY);
         At = COMP(A1,At);
         }
       At = INV(At);
       As = At;

Step2: /* Root cell. */
       if (k > 0) goto Step3;
    
                             /*Int*/ Ths = ACLOCK();
       SLELTI(c,DEGSUB,PLDEG(Ps));

       if (Ps != NIL) I = IPLRRI(Ps); else I = NIL;
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMIPLRRI[k + 1] = TMIPLRRI[k + 1] + Ths;
                          
                             /*Int*/ Ths = ACLOCK();
       EC1(c,I,Ps);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMECR[k + 1] = TMECR[k + 1] + Ths;
                          
                             /*Int*/ Ths = ACLOCK();
       if (As != NIL || 1 <= f) SIGNP1(c,Ps,I);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMSIGNPR[k + 1] = TMSIGNPR[k + 1] + Ths;
                             /*Int*/ NMSTACK[k + 1] = NMSTACK[k + 1] + 1;
                             /*Int*/ NMSTACKR[k + 1] = NMSTACKR[k + 1] + 1;
                             /*Int*/ NMCELL[k + 1] = NMCELL[k + 1] + LENGTH(LELTI(c,CHILD));
    
       goto Return;

Step3: /* Non-root cell, Irrational sample point. */
       s = LELTI(c,SAMPLE); FIRST3(s,&M,&I,&b);
    
       if (PDEG(M) == 1      
                             /*Int*/ && PCRSP == 'y'
          ) goto Step4;
    
                             /*Int*/ Ths = ACLOCK();
       S = SUBST(k,M,b,Ps);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMSUBST[k + 1] = TMSUBST[k + 1] + Ths;
                             /*Int*/ if (LSRCH('l',PCPROJOP) != 0 && LSRCH(0,S) != 0)
                             /*Int*/   FAIL("CONSTRUCT","Substitution failed",k,M,b,Ps,S);

       SLELTI(c,DEGSUB,PLDEG(S));
      
                             /*Int*/ Ths = ACLOCK();
       AFUPLM(M,S,&L,&T);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMAFUPLM[k + 1] = TMAFUPLM[k + 1] + Ths;
                          
                             /*Int*/ Ths = ACLOCK();
       AFCSBMDB(M,T,&B,&E);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMAFCSBM[k + 1] = TMAFCSBM[k + 1] + Ths;
                          
                             /*Int*/ Ths = ACLOCK();
       if (B != NIL) I = AFUPBRI(M,I,B); else I = NIL;
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMAFPBRI[k + 1] = TMAFPBRI[k + 1] + Ths;
                          
                             /*Int*/ Ths = ACLOCK();
       EC(c,I,E,B);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMEC[k + 1] = TMEC[k + 1] + Ths;
                          
                             /*Int*/ Ths = ACLOCK();
       if (As != NIL || k + 1 <= f) SIGNP(c,k,B,I,E,L);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMSIGNP[k + 1] = TMSIGNP[k + 1] + Ths;
                             /*Int*/ NMSTACK[k + 1] = NMSTACK[k + 1] + 1;
                             /*Int*/ NMCELL[k + 1] = NMCELL[k + 1] + LENGTH(LELTI(c,CHILD)); 
       goto Return;

Step4: /* Non-root cell, Rational sample point. */
                             /*Int*/ Ths = ACLOCK();
       b = RCFAFC(b);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMRCFAFC[k + 1] = TMRCFAFC[k + 1] + Ths;
                          
                             /*Int*/ Ths = ACLOCK();
       S = SUBSTR(k,b,Ps);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMSUBSTR[k + 1] = TMSUBSTR[k + 1] + Ths;
                             /*Int*/ if (LSRCH('l',PCPROJOP) != 0 && LSRCH(0,S) != 0)
                             /*Int*/   FAIL("CONSTRUCT","Substitution failed",k,M,b,Ps,S);
       SLELTI(c,DEGSUB,PLDEG(S));
                          
                             /*Int*/ Ths = ACLOCK();
       IPLSRP(S,&s,&T);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMIPLSRP[k + 1] = TMIPLSRP[k + 1] + Ths;
                          
                             /*Int*/ Ths = ACLOCK();
       IPFSBM(1,T,&B,&E);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMIPFSBM[k + 1] = TMIPFSBM[k + 1] + Ths;
                          
                             /*Int*/ Ths = ACLOCK();
       if (B != NIL) I = IPLRRI(B); else I = NIL;
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMIPLRRI[k + 1] = TMIPLRRI[k + 1] + Ths;
                          
                             /*Int*/ Ths = ACLOCK();
       ECR(c,I,E,B);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMECR[k + 1] = TMECR[k + 1] + Ths;
                          
                             /*Int*/ Ths = ACLOCK();
       if (As != NIL || k + 1 <= f) SIGNPR(c,k,B,I,E,s);
                             /*Int*/ Ths = ACLOCK() - Ths;
                             /*Int*/ TMSIGNPR[k + 1] = TMSIGNPR[k + 1] + Ths;
                             /*Int*/ NMSTACK[k + 1] = NMSTACK[k + 1] + 1;
                             /*Int*/ NMSTACKR[k + 1] = NMSTACKR[k + 1] + 1;
                             /*Int*/ NMCELL[k + 1] = NMCELL[k + 1] +  LENGTH(LELTI(c,CHILD));

Return: /* Prepare for return. */
       return;
}




